@@ -32,10 +32,11 @@
                                scattering, nstemp, int_mob, scissor, carrier,      &
                                iterative_bte, longrange, scatread, nqf1, prtgkk,   &
                                nqf2, nqf3, mp_mesh_k, restart, plselfen,           &
-                               specfun_pl, lindabs, use_ws, epbread,               &
+                               specfun_pl, lindabs, use_ws, epbread, fermi_plot,   &
                                epmatkqread, selecqread, restart_step, nsmear,      &
                                nqc1, nqc2, nqc3, nkc1, nkc2, nkc3, assume_metal,   &
-                               cumulant, eliashberg
+                               cumulant, eliashberg, nomega,                       &
+                               omegamin, omegamax, omegastep, neta
   USE control_flags,    ONLY : iverbosity
   USE noncollin_module, ONLY : noncolin
   USE constants_epw,    ONLY : ryd2ev, ryd2mev, one, two, zero, czero, eps40,      &
@@ -55,7 +56,10 @@
                                inv_tau_allcb, zi_allcb, exband, gamma_v_all,       &
                                esigmar_all, esigmai_all, lower_bnd, upper_bnd,     &
                                a_all, a_all_ph, wscache, lambda_v_all, threshold,  &
-                               nktotf, gtemp, xkq, dos, nbndskip, nbndep
+                               nktotf, gtemp, xkq, dos, nbndskip, nbndep,          &
+                               inv_tau_all_mode, inv_tau_allcb_mode, qrpl, Qmat,   &
+                               ef0_fca, epsilon2_abs, epsilon2_abs_lorenz,         &
+                               epsilon2_abs_all, epsilon2_abs_lorenz_all
   USE wan2bloch,        ONLY : dmewan2bloch, hamwan2bloch, dynwan2bloch,           &
                                ephwan2blochp, ephwan2bloch, vmewan2bloch,          &
                                dynifc2blochf, vmewan2blochp
@@ -66,8 +70,8 @@
   USE io_eliashberg,    ONLY : write_ephmat, count_kpoints, kmesh_fine, kqmap_fine,&
                                check_restart_ephwrite
   USE transport,        ONLY : transport_coeffs, scattering_rate_q
-  USE grid,             ONLY : qwindow
-  USE printing,         ONLY : print_gkk, plot_band
+  USE grid,             ONLY : qwindow, loadkmesh_fst, xqf_otf
+  USE printing,         ONLY : print_gkk, plot_band, plot_fermisurface
   USE io_epw,           ONLY : rwepmatw, epw_read, epw_write
   USE io_transport,     ONLY : tau_read, iter_open, print_ibte, iter_merge
   USE io_selfen,        ONLY : selfen_el_read, spectral_read
@@ -85,8 +89,10 @@
                                nesting_fn_q
   USE spectral_func,    ONLY : spectral_func_el_q, spectral_func_ph_q, a2f_main,   &
                                spectral_func_pl_q
-  USE rigid_epw,        ONLY : rpa_epsilon, tf_epsilon, compute_umn_f, rgd_blk_epw_fine
-  USE indabs,           ONLY : indabs_main, renorm_eig
+  USE rigid_epw,        ONLY : rpa_epsilon, tf_epsilon, compute_umn_f, rgd_blk_epw_fine !, &
+!                               find_gmin ! Temporarily commented by H. Lee
+  USE indabs,           ONLY : indabs_main, renorm_eig, fermi_carrier_indabs
+  USE io_indabs,        ONLY : indabs_read
 #if defined(__MPI)
   USE parallel_include, ONLY : MPI_MODE_RDONLY, MPI_INFO_NULL, MPI_OFFSET_KIND, &
                                MPI_OFFSET
@@ -142,6 +148,8 @@
   !! Counter on coarse q-point grid
   INTEGER :: iq_restart
   !! Counter on coarse q-point grid
+  INTEGER :: qind
+  !! Index of q-point either on full q-grid or within qwindow
   INTEGER :: ik
   !! Counter on coarse k-point grid
   INTEGER :: ikk
@@ -209,7 +217,7 @@
   !! Wigner-Seitz number of degenerescence (weights) for the electrons grid
   INTEGER, ALLOCATABLE :: ndegen_q(:, :, :)
   !! Wigner-Seitz weights for the phonon grid that depend on atomic positions $R + \tau(nb) - \tau(na)$
-  INTEGER, ALLOCATABLE :: ndegen_g(:, :, :, :)
+  INTEGER, ALLOCATABLE :: ndegen_g(:, :, :)
   !! Wigner-Seitz weights for the electron-phonon grid that depend on
   !! atomic positions $R - \tau(na)$
   INTEGER, ALLOCATABLE :: selecq(:)
@@ -251,6 +259,8 @@
   !! Maximum vector: at*nq
   REAL(KIND = DP) :: etemp
   !! Temperature in Ry (this includes division by kb)
+  REAL(KIND = DP) :: etemp_fca
+  !! Temperature for free carrier absorption
   REAL(KIND = DP) :: ef0(nstemp)
   !! Fermi level for the temperature itemp
   REAL(KIND = DP) :: efcb(nstemp)
@@ -336,6 +346,10 @@
     WRITE(stdout, '(/,5x,a)' ) 'Computes the analytic long-range interaction for polar materials [lpolar]'
     WRITE(stdout, '(5x,a)' )   ' '
   ENDIF
+  IF (.NOT. lpolar .AND. qrpl) THEN
+    WRITE(stdout, '(/,5x,a)' ) 'Computes the analytic quadrupole long-range interaction for non-polar materials [Q1,Q2]'
+    WRITE(stdout, '(5x,a)' )   ' '
+  ENDIF
   !
   ! Determine Wigner-Seitz points
   !
@@ -408,7 +422,7 @@
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating chw_ks', 1)
     ALLOCATE(rdw(nmodes, nmodes, nrr_q), STAT = ierr)
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating rdw', 1)
-    IF (vme) THEN
+    IF (vme == 'wannier') THEN
       ALLOCATE(cvmew(3, nbndsub, nbndsub, nrr_k), STAT = ierr)
       IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating cvmew', 1)
       cvmew(:, :, :, :) = czero
@@ -443,7 +457,7 @@
       CALL hambloch2wan(nbnd, nbndsub, nks, nkstot, et_ks, xk_loc, cu, lwin, exband, nrr_k, irvec_k, wslen_k, chw_ks)
     ENDIF
     !
-    IF (vme) THEN
+    IF (vme == 'wannier') THEN
       ! Transform of position matrix elements
       ! PRB 74 195118  (2006)
       CALL vmebloch2wan(nbnd, nbndsub, nks, nkstot, xk_loc, cu, nrr_k, irvec_k, wslen_k, lwin, exband)
@@ -460,9 +474,8 @@
     ! Electron-Phonon vertex (Bloch el and Bloch ph -> Wannier el and Bloch ph)
     !
     ! Open the prefix.epmatwe file
-    IF ((etf_mem == 1) .AND. ionode) THEN
+    IF ((etf_mem == 1 .OR. etf_mem == 3) .AND. ionode) THEN
       lrepmatw = 2 * nbndsub * nbndsub * nrr_k * nmodes
-      filint   = TRIM(prefix)//'.epmatwe'
       CALL diropn(iunepmatwe, 'epmatwe', lrepmatw, exst)
     ENDIF
     !
@@ -493,7 +506,7 @@
         !
       ENDDO
       ! Only the master node writes
-      IF ((etf_mem == 1) .AND. (ionode)) THEN
+      IF ((etf_mem == 1 .OR. etf_mem == 3) .AND. ionode) THEN
         ! direct write of epmatwe for this iq
         CALL rwepmatw(epmatwe_mem, nbndsub, nrr_k, nmodes, iq, iunepmatwe, +1)
         !
@@ -502,7 +515,7 @@
     ENDDO
     WRITE(stdout, '(a)' ) ' '
     !
-    IF (etf_mem == 1 .AND. ionode) CLOSE(iunepmatwe, STATUS = 'keep')
+    IF ((etf_mem == 1 .OR. etf_mem == 3) .AND. ionode) CLOSE(iunepmatwe, STATUS = 'keep')
     !
     ! Electron-Phonon vertex (Wannier el and Bloch ph -> Wannier el and Wannier ph)
     !
@@ -522,7 +535,7 @@
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating epmatq', 1)
     DEALLOCATE(dynq, STAT = ierr)
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating dynq', 1)
-    IF (.NOT. vme) DEALLOCATE(dmec)
+    IF (vme == 'dipole') DEALLOCATE(dmec)
     IF (etf_mem == 0) THEN
       DEALLOCATE(epmatwe, STAT = ierr)
       IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating epmatwe', 1)
@@ -554,17 +567,27 @@
   ! At this point, we will interpolate the Wannier rep to the Bloch rep
   ! for electrons, phonons and the ep-matrix
   !
-  ! Load the fine-grid q and k grids.
-  ! nkqtotf is computed inside
-  CALL loadqmesh_serial
-  CALL loadkmesh_para
+  ! Load the fine-grid q and k grids (nkqtotf is computed inside).
+  ! The k-point mapping between BZ and IBZ is allocated and computed here as well bztoibz and s_bztoibz
+  ! etf_mem == 3 is a special optimization level to deal with ultra dense fine homogeneous grids.
+  IF (etf_mem == 3) THEN
+    ! The fine q-grid is not create (will be generated on the fly later)
+    ! The fine k-grid consists of point within the fsthick window using k-point symmetry
+    CALL loadkmesh_fst(nrr_k, dims, irvec_k, ndegen_k, nelec)
+    nqtotf = nqf1 * nqf2 * nqf3
+    nqf = nqtotf
+  ELSE
+    CALL loadqmesh_serial()
+    CALL loadkmesh_para()
+    ALLOCATE(wf(nmodes, nqtotf), STAT = ierr)
+    IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating wf', 1)
+    wf(:, :) = zero
+  ENDIF
   ! Defines the total number of k-points
   nktotf = nkqtotf / 2
   !
   ALLOCATE(epmatwef(nbndsub, nbndsub, nrr_k, nmodes), STAT = ierr)
   IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating epmatwef', 1)
-  ALLOCATE(wf(nmodes, nqf), STAT = ierr)
-  IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating wf', 1)
   ALLOCATE(etf(nbndsub, nkqf), STAT = ierr)
   IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating etf', 1)
   ALLOCATE(etf_ks(nbndsub, nkqf), STAT = ierr)
@@ -584,7 +607,6 @@
   ALLOCATE(isk_dummy(nkqf), STAT = ierr)
   IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating isk_dummy', 1)
   epmatwef(:, :, :, :) = czero
-  wf(:, :)             = zero
   etf(:, :)            = zero
   etf_ks(:, :)         = zero
   epmatf(:, :, :)      = czero
@@ -596,8 +618,7 @@
   isk_dummy(:)         = 0
   !
   ! Allocate velocity and dipole matrix elements after getting grid size
-  !
-  IF (vme) THEN
+  IF (vme == 'wannier') THEN
     ALLOCATE(vmef(3, nbndsub, nbndsub, 2 * nkf), STAT = ierr)
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating vmef', 1)
     vmef(:, :, :, :) = czero
@@ -760,9 +781,9 @@
   ! Define it only once for the full run.
   CALL fkbounds(nktotf, lower_bnd, upper_bnd)
   !
-  ! Re-order the k-point according to weather they are in or out of the fshick
-  ! windows
-  IF ((iterative_bte .OR. ephwrite) .AND. mp_mesh_k) THEN
+  ! Re-order the k-point according to weather they are in or out of the fshick windows
+  ! In the case of etf_mem == 3, we only have k-points within the fsthick (no need for re-ordering).
+  IF ((iterative_bte .OR. ephwrite) .AND. mp_mesh_k .AND. etf_mem < 3) THEN
     CALL load_rebal
   ENDIF
   !
@@ -786,7 +807,7 @@
   !
   ! Open the ephmatwp file here
 #if defined(__MPI)
-  IF (etf_mem == 1) then
+  IF (etf_mem == 1 .OR. etf_mem == 3) then
     ! Check for directory given by "outdir"
     !
     filint = TRIM(tmp_dir) // TRIM(prefix)//'.epmatwp'
@@ -795,7 +816,7 @@
   ENDIF
 #else
   lrepmatw = 2 * nbndsub * nbndsub * nrr_k * nmodes
-  filint   = TRIM(prefix)//'.epmatwp'
+  filint = TRIM(tmp_dir) // TRIM(prefix)//'.epmatwp'
   INQUIRE(IOLENGTH = direct_io_factor) dummy(1)
   unf_recl = direct_io_factor * INT(lrepmatw, KIND = KIND(unf_recl))
   IF (unf_recl <= 0) CALL errore('epw_write', 'wrong record length', 3)
@@ -829,6 +850,14 @@
     inv_tau_allcb(:, :, :) = zero
     lrepmatw2_restart(:)   = 0
     lrepmatw5_restart(:)   = 0
+    IF (iverbosity == 3) THEN
+      ALLOCATE(inv_tau_all_mode(nmodes, nbndfst, nktotf, nstemp), STAT = ierr)
+      IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating inv_tau_all_mode', 1)
+      ALLOCATE(inv_tau_allcb_mode(nmodes, nbndfst, nktotf, nstemp), STAT = ierr)
+      IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating inv_tau_allcb_mode', 1)
+      inv_tau_all_mode(:, :, :, :)   = zero
+      inv_tau_allcb_mode(:, :, :, :) = zero
+    ENDIF
     ! We save matrix elements that are smaller than machine precision (1d-16).
     ! The sum of all the elements must be smaller than that
     ! nkf1 * nkf2 * nkf3 * nqf1 * nqf2 * nqf3 * (nbndfst) * (nbndfst)
@@ -861,6 +890,12 @@
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating inv_tau_all', 1)
     DEALLOCATE(inv_tau_allcb, STAT = ierr)
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating inv_tau_allcb', 1)
+    IF (iverbosity == 3) THEN
+      DEALLOCATE(inv_tau_all_mode, STAT = ierr)
+      IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating inv_tau_all_mode', 1)
+      DEALLOCATE(inv_tau_allcb_mode, STAT = ierr)
+      IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating inv_tau_allcb_mode', 1)
+    ENDIF
     !
   ELSE ! (iterative_bte .AND. epmatkqread)
     IF (iterative_bte) THEN
@@ -942,6 +977,12 @@
     ! Fine mesh set of g-matrices.  It is large for memory storage
     ALLOCATE(epf17(nbndfst, nbndfst, nmodes, nkf), STAT = ierr)
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating epf17', 1)
+    ! We allocate the phonon frequency on the q-points within the window
+    IF (etf_mem == 3) THEN
+      ALLOCATE(wf(nmodes, totq), STAT = ierr)
+      IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating wf', 1)
+      wf(:, :) = zero
+    ENDIF
     IF (phonselfen) THEN
       ALLOCATE(lambda_all(nmodes, totq, nsmear, nstemp), STAT = ierr)
       IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating lambda_all', 1)
@@ -1004,6 +1045,18 @@
         sigmai_mode(:, :, :, :) = zero
       ENDIF
     ENDIF ! elecselfen
+    IF (lindabs) THEN
+      ! Calculate the number of frequency points
+      nomega = INT((omegamax - omegamin) / omegastep) + 1
+      ALLOCATE(epsilon2_abs(3, nomega, neta, nstemp), STAT = ierr)
+      IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating epsilon2_abs', 1)
+      ALLOCATE(epsilon2_abs_lorenz(3, nomega, neta, nstemp), STAT = ierr)
+      IF (ierr /= 0) CALL errore('ephwann_shiffle', 'Error allocating epsilon2_abs_lorenz', 1)
+      ALLOCATE(epsilon2_abs_all(3, nomega, neta, nstemp), STAT = ierr)
+      IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating epsilon2_abs_all', 1)
+      ALLOCATE(epsilon2_abs_lorenz_all(3, nomega, neta, nstemp), STAT = ierr)
+      IF (ierr /= 0) CALL errore('ephwann_shiffle', 'Error allocating epsilon2_abs_lorenz_all', 1)
+    ENDIF ! indabs
     !
     ! --------------------------------------------------------------------------------------
     ! Polaron shell implementation for future use
@@ -1056,6 +1109,10 @@
         ENDIF
       ENDIF
       !
+      ! Restart in indirect optics
+      IF (lindabs) THEN
+        CALL indabs_read(iq_restart, totq, epsilon2_abs_all, epsilon2_abs_lorenz_all)
+      ENDIF
       ! If you restart from reading a file. This prevent
       ! the case were you restart but the file does not exist
       IF (iq_restart > 1) first_cycle = .TRUE.
@@ -1093,7 +1150,7 @@
         CALL mp_bcast(lrepmatw2_restart, ionode_id, world_comm)
         CALL mp_bcast(lrepmatw5_restart, ionode_id, world_comm)
         IF (npool /= npool_tmp) CALL errore('ephwann_shuffle','Number of cores is different',1)
-        ! 
+        !
         IF (iterative_bte) THEN
           IF (mpime == ionode_id) THEN
             OPEN(UNIT = iuntau, FORM = 'unformatted', FILE = 'inv_tau_tmp', STATUS = 'old')
@@ -1160,6 +1217,11 @@
       !
       iq = selecq(iqq)
       !
+      IF (etf_mem == 3) THEN
+        qind = iqq
+      ELSE
+        qind = iq
+      ENDIF
       !
       ! In case of big calculation, show progression of iq (especially usefull when
       ! elecselfen = true as nothing happen during the calculation otherwise.
@@ -1170,7 +1232,15 @@
         ENDIF
       ENDIF
       !
-      xxq = xqf(:, iq)
+      ! xqf has to be in crystal coordinate
+      IF (etf_mem == 3) THEN
+        ! The q-point coordinate is generate on the fly for each q-point
+        CALL xqf_otf(iq, xxq)
+      ELSE
+        xxq = xqf(:, iq)
+      ENDIF
+      ! Temporarily commented by H. Lee
+!      CALL find_gmin(xxq)
       !
       ! ------------------------------------------------------
       ! dynamical matrix : Wannier -> Bloch
@@ -1190,9 +1260,9 @@
         ! (omega on fine grid)
         !
         IF (w2(nu) > -eps8) THEN
-          wf(nu, iq) =  DSQRT(ABS(w2(nu)))
+          wf(nu, qind) =  DSQRT(ABS(w2(nu)))
         ELSE
-          wf(nu, iq) = -DSQRT(ABS(w2(nu)))
+          wf(nu, qind) = -DSQRT(ABS(w2(nu)))
         ENDIF
         !
         DO mu = 1, nmodes
@@ -1213,7 +1283,7 @@
       fermicount = 0
       !
       IF (lscreen) THEN
-        IF (scr_typ == 0) CALL rpa_epsilon(xxq, wf(:, iq), nmodes, epsi, eps_rpa)
+        IF (scr_typ == 0) CALL rpa_epsilon(xxq, wf(:, qind), nmodes, epsi, eps_rpa)
         IF (scr_typ == 1) CALL tf_epsilon(xxq, nmodes, epsi, eps_rpa)
       ENDIF
       !
@@ -1222,7 +1292,7 @@
       ! -------------------------------------------------------------
       !
       IF (adapt_smearing) THEN
-        CALL vmewan2blochp(xxq, nmodes, nrr_q, irvec_q, ndegen_q, uf, vmefp(:, :, :), wf(:, iq), rws, nrws)
+        CALL vmewan2blochp(xxq, nmodes, nrr_q, irvec_q, ndegen_q, uf, vmefp(:, :, :), wf(:, qind), rws, nrws)
       ENDIF
       !
       ! This is a loop over k blocks in the pool (size of the local k-set)
@@ -1273,30 +1343,30 @@
         etf(icbm:nbndsub, ikq) = etf(icbm:nbndsub, ikq) + scissor
         !
         IF (.NOT. scattering) THEN
-          IF (vme) THEN
-             !
-             ! ------------------------------------------------------
-             !  velocity: Wannier -> Bloch
-             ! ------------------------------------------------------
-             !
-             IF (eig_read) THEN
-               !
-               ! Renormalize the eigenvalues and vmef with the read eigenvalues
-               CALL renorm_eig(ikk, ikq, nrr_k, dims, ndegen_k, irvec_k, irvec_r, cufkk, cufkq, cfac, cfacq)
-               !
-             ELSE ! eig_read
-               CALL vmewan2bloch(nbndsub, nrr_k, irvec_k, cufkk, vmef(:, :, :, ikk), etf(:, ikk), etf_ks(:, ikk), chw, cfac, dims)
-               CALL vmewan2bloch(nbndsub, nrr_k, irvec_k, cufkq, vmef(:, :, :, ikq), etf(:, ikq), etf_ks(:, ikq), chw, cfacq, dims)
-             ENDIF
+          IF (vme == 'wannier') THEN
+            !
+            ! ------------------------------------------------------
+            !  velocity: Wannier -> Bloch
+            ! ------------------------------------------------------
+            !
+            IF (eig_read) THEN
+              !
+              ! Renormalize the eigenvalues and vmef with the read eigenvalues
+              CALL renorm_eig(ikk, ikq, nrr_k, dims, ndegen_k, irvec_k, irvec_r, cufkk, cufkq, cfac, cfacq)
+              !
+            ELSE ! eig_read
+              CALL vmewan2bloch(nbndsub, nrr_k, irvec_k, cufkk, vmef(:, :, :, ikk), etf(:, ikk), etf_ks(:, ikk), chw, cfac, dims)
+              CALL vmewan2bloch(nbndsub, nrr_k, irvec_k, cufkq, vmef(:, :, :, ikq), etf(:, ikq), etf_ks(:, ikq), chw, cfacq, dims)
+            ENDIF
           ELSE
-             !
-             ! ------------------------------------------------------
-             !  dipole: Wannier -> Bloch
-             ! ------------------------------------------------------
-             !
-             CALL dmewan2bloch(nbndsub, nrr_k, cufkk, dmef(:, :, :, ikk), etf(:, ikk), etf_ks(:, ikk), cfac, dims)
-             CALL dmewan2bloch(nbndsub, nrr_k, cufkq, dmef(:, :, :, ikq), etf(:, ikq), etf_ks(:, ikq), cfacq, dims)
-             !
+            !
+            ! ------------------------------------------------------
+            !  dipole: Wannier -> Bloch
+            ! ------------------------------------------------------
+            !
+            CALL dmewan2bloch(nbndsub, nrr_k, cufkk, dmef(:, :, :, ikk), etf(:, ikk), etf_ks(:, ikk), cfac, dims)
+            CALL dmewan2bloch(nbndsub, nrr_k, cufkq, dmef(:, :, :, ikq), etf(:, ikq), etf_ks(:, ikq), cfacq, dims)
+            !
           ENDIF
         ENDIF
         !
@@ -1310,7 +1380,7 @@
             ! Compute velocities
             !
             IF (scattering) THEN
-              IF (vme) THEN
+              IF (vme == 'wannier') THEN
                 CALL vmewan2bloch &
                      (nbndsub, nrr_k, irvec_k, cufkk, vmef(:, :, :, ikk), etf(:, ikk), etf_ks(:, ikk), chw, cfac, dims)
                 CALL vmewan2bloch &
@@ -1327,7 +1397,7 @@
             !
             IF (adapt_smearing) THEN
               ! Return the value of the adaptative broadening eta
-              CALL broadening(ik, ikk, ikq, wf(:, iq), vmefp, eta)
+              CALL broadening(ik, ikk, ikq, wf(:, qind), vmefp, eta)
               !
             ENDIF ! adapt_smearing
             !
@@ -1352,7 +1422,7 @@
               !
             ENDIF
             !
-            IF (lpolar) THEN
+            IF (lpolar .OR. qrpl) THEN
               !
               CALL compute_umn_f(nbndsub, cufkk, cufkq, bmatf)
               !
@@ -1418,11 +1488,11 @@
       IF (prtgkk    ) CALL print_gkk(iq)
       IF (phonselfen) CALL selfen_phon_q(iqq, iq, totq)
       IF (elecselfen) CALL selfen_elec_q(iqq, iq, totq, first_cycle)
-      IF (plselfen .AND. .NOT. vme) CALL selfen_pl_q(iqq, iq, totq, first_cycle)
+      IF (plselfen .AND. vme == 'dipole') CALL selfen_pl_q(iqq, iq, totq, first_cycle)
       IF (nest_fn   ) CALL nesting_fn_q(iqq, iq)
       IF (specfun_el) CALL spectral_func_el_q(iqq, iq, totq, first_cycle)
       IF (specfun_ph) CALL spectral_func_ph_q(iqq, iq, totq)
-      IF (specfun_pl .AND. .NOT. vme) CALL spectral_func_pl_q(iqq, iq, totq, first_cycle)
+      IF (specfun_pl .AND. vme == 'dipole') CALL spectral_func_pl_q(iqq, iq, totq, first_cycle)
       IF (ephwrite) THEN
         IF (first_cycle .OR. iq == 1) THEN
            CALL kmesh_fine
@@ -1444,7 +1514,17 @@
         ENDIF
         !
         ! Indirect absorption
-        IF (lindabs .AND. .NOT. scattering)  CALL indabs_main(iq)
+        IF (lindabs .AND. .NOT. scattering) THEN
+          IF (carrier .and. (iq == iq_restart)) THEN
+            ALLOCATE(ef0_fca(nstemp), STAT = ierr)
+            IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error allocating ef0_fca', 1)
+            DO itemp = 1, nstemp
+              etemp_fca = gtemp(itemp)
+              CALL fermi_carrier_indabs(itemp, etemp_fca, ef0_fca)
+            ENDDO
+          ENDIF
+          CALL indabs_main(iq, totq, first_cycle, iq_restart)
+        ENDIF
         !
         ! Conductivity ---------------------------------------------------------
         IF (scattering) THEN
@@ -1462,7 +1542,7 @@
             ENDDO
           ENDIF
           !
-          IF (.NOT. iterative_bte) THEN
+          IF (.NOT. iterative_bte .AND. etf_mem < 3) THEN
             CALL scattering_rate_q(iqq, iq, totq, ef0, efcb, first_cycle)
             ! Computes the SERTA mobility
             IF (iqq == totq) CALL transport_coeffs(ef0, efcb)
@@ -1470,7 +1550,7 @@
           !
           IF (iterative_bte) THEN
             CALL start_clock('print_ibte')
-            CALL print_ibte(iqq, iq, totq, ef0, efcb, first_cycle, ind_tot, ind_totcb, &
+            CALL print_ibte(iqq, iq, totq, xxq, ef0, efcb, first_cycle, ind_tot, ind_totcb, &
                             lrepmatw2_restart, lrepmatw5_restart, ctype)
             CALL stop_clock('print_ibte')
             !
@@ -1612,6 +1692,8 @@
     ENDIF
     IF (band_plot) CALL plot_band()
     !
+    IF (fermi_plot) CALL plot_fermisurface()
+    !
     IF (a2f) CALL a2f_main()
     !
     ! if scattering is read then Fermi level and scissor have not been computed.
@@ -1730,13 +1812,13 @@
     !
   ENDIF ! (iterative_bte .AND. epmatkqread)
   !
-  IF ((iterative_bte .OR. ephwrite) .AND. mp_mesh_k) THEN
+  IF ((iterative_bte .OR. ephwrite) .AND. mp_mesh_k .AND. etf_mem < 3) THEN
     DEALLOCATE(map_rebal, STAT = ierr)
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating map_rebal', 1)
     DEALLOCATE(map_rebal_inv, STAT = ierr)
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating map_rebal_inv', 1)
   ENDIF
-  IF (vme) THEN
+  IF (vme == 'wannier') THEN
     DEALLOCATE(vmef, STAT = ierr)
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating vmef', 1)
     DEALLOCATE(cvmew, STAT = ierr)
@@ -1764,8 +1846,10 @@
   IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating zstar', 1)
   DEALLOCATE(epmatwef, STAT = ierr)
   IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating epmatwef', 1)
-  DEALLOCATE(wf, STAT = ierr)
-  IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating wf', 1)
+  IF (.NOT. epmatkqread) THEN
+    DEALLOCATE(wf, STAT = ierr)
+    IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating wf', 1)
+  ENDIF
   DEALLOCATE(etf, STAT = ierr)
   IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating etf', 1)
   DEALLOCATE(etf_ks, STAT = ierr)
@@ -1821,13 +1905,29 @@
     DEALLOCATE(gtemp, STAT = ierr)
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating gtemp', 1)
   ENDIF
-  !
   DEALLOCATE(et_ks, STAT = ierr)
   IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating et_ks', 1)
   IF (assume_metal) THEN
     DEALLOCATE(dos, STAT = ierr)
     IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating dos', 1)
   ENDIF
+  DEALLOCATE(Qmat, STAT = ierr)
+  IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating Qmat', 1)
+  !
+  IF (lindabs .AND. (.NOT. scattering)) THEN
+    IF (carrier) THEN
+      DEALLOCATE(ef0_fca, STAT = ierr)
+      IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating ef0_fca', 1)
+    ENDIF
+    DEALLOCATE(epsilon2_abs, STAT = ierr)
+    IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating epsilon2_abs', 1)
+    DEALLOCATE(epsilon2_abs_lorenz, STAT = ierr)
+    IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating epsilon2_abs_lorenz', 1)
+    DEALLOCATE(epsilon2_abs_all, STAT = ierr)
+    IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating epsilon2_abs_all', 1)
+    DEALLOCATE(epsilon2_abs_lorenz_all, STAT = ierr)
+    IF (ierr /= 0) CALL errore('ephwann_shuffle', 'Error deallocating epsilon2_abs_lorenz_all', 1)
+  ENDIF
   !
   CALL stop_clock('ephwann')
   !
