@@ -12,13 +12,13 @@
   !-----------------------------------------------------------------------
   !!
   !! This routine reads the control variables for the program epw.
-  !! from standard input (unit 5).
-  !! A second routine readfile reads the variables saved on a file
+  !! A second routine, read_file, reads the variables saved on a file
   !! by the self-consistent program.
   !!
   !! @Note:
   !!   SP: Image parallelization added
   !!
+  USE kinds,         ONLY : DP
   USE ions_base,     ONLY : nat, ntyp => nsp
   USE cell_base,     ONLY : at
   USE mp,            ONLY : mp_bcast
@@ -48,7 +48,7 @@
                             laniso, lpolar, lifc, asr_typ, lscreen, scr_typ, nbndsub,  &
                             fermi_diff, smear_rpa, cumulant, bnd_cum, proj, write_wfn, &
                             iswitch, liso, lacon, lpade, etf_mem, epbwrite,            &
-                            nsiter, conv_thr_racon, specfun_el, specfun_ph,            &
+                            nsiter, conv_thr_racon, npade, specfun_el, specfun_ph,     &
                             system_2d, delta_approx, title, int_mob, scissor,          &
                             iterative_bte, scattering, selecqread, epmatkqread,        &
                             ncarrier, carrier, scattering_serta, restart, restart_step,&
@@ -58,28 +58,28 @@
                             auto_projections, scdm_proj, scdm_entanglement, scdm_mu,   &
                             scdm_sigma, assume_metal, wannier_plot, wannier_plot_list, &
                             wannier_plot_supercell, wannier_plot_scale, reduce_unk,    &
-                            wannier_plot_radius,                                       &
-                            fixsym, epw_no_t_rev, epw_tr, epw_nosym, epw_noinv,        &
-                            epw_crysym
+                            wannier_plot_radius, fermi_plot, fixsym, epw_no_t_rev,     &
+                            epw_tr, epw_nosym, epw_noinv, epw_crysym,                  &
+                            bfieldx, bfieldy, bfieldz, tc_linear, tc_linear_solver
   USE klist_epw,     ONLY : xk_all, xk_loc, xk_cryst, isk_all, isk_loc, et_all, et_loc
   USE elph2,         ONLY : elph, num_wannier_plot, wanplotlist, gtemp
   USE constants_epw, ONLY : ryd2mev, ryd2ev, ev2cmm1, kelvin2eV, zero, eps20, ang2m
+  USE constants,     ONLY : electron_si
   USE io_files,      ONLY : tmp_dir, prefix
   USE control_flags, ONLY : iverbosity, modenum, gamma_only
   USE ions_base,     ONLY : amass
-  USE mp_world,      ONLY : world_comm, mpime
+  USE mp_world,      ONLY : world_comm, mpime, nproc
   USE partial,       ONLY : atomo, nat_todo
   USE constants,     ONLY : AMU_RY, eps16
-  USE mp_global,     ONLY : my_pool_id, me_pool
-  USE io_global,     ONLY : meta_ionode, meta_ionode_id, stdout
+  USE mp_pools,      ONLY : my_pool_id, me_pool, npool
+  USE mp_images,     ONLY : nimage
+  USE io_global,     ONLY : meta_ionode, meta_ionode_id, qestdin, stdout
   USE io_var,        ONLY : iunkf, iunqf
   USE noncollin_module, ONLY : npol, noncolin
   USE wvfct,         ONLY : npwx
   USE paw_variables, ONLY : okpaw
   USE io_epw,        ONLY : param_get_range_vector
-#if defined(__NAG)
-  USE F90_UNIX_ENV,  ONLY : iargc, getarg
-#endif
+  USE open_close_input_file, ONLY : open_input_file, close_input_file
   !
   ! ---------------------------------------------------------------------------------------
   ! Added for polaron calculations. Originally by Danny Sio, modified by Chao Lian.
@@ -103,9 +103,7 @@
   !! Output directory
   CHARACTER(LEN = 512) :: line
   !! Line in input file
-#if ! defined(__NAG)
-  INTEGER :: iargc
-#endif
+  CHARACTER(LEN=256), EXTERNAL :: trimcheck
   INTEGER :: ios
   !! INTEGER variable for I/O control
   INTEGER :: ios2
@@ -134,27 +132,24 @@
   !! Error status
   INTEGER :: unit_loc = 5
   !! Unit for input file
+  REAL(kind = DP) :: b_abs
+  !! Absolute magnetic field
   !
   NAMELIST / inputepw / &
-       amass, outdir, prefix, iverbosity, fildvscf,                            &
-       elph, nq1, nq2, nq3, nk1, nk2, nk3, nbndsub,                            &
-       filukk, epbread, epbwrite, epwread, epwwrite, etf_mem, kmaps,           &
-       eig_read, wepexst, epexst, vme,                                         &
-       degaussw, fsthick, nsmear, delta_smear,                                 &
-       dvscf_dir, ngaussw, epmatkqread, selecqread,                            &
-       wannierize, dis_win_max, dis_win_min, dis_froz_min, dis_froz_max,       &
-       num_iter, proj, bands_skipped, wdata, iprint, write_wfn,                &
-       wmin, wmax, nw, eps_acustic, a2f, nest_fn, plselfen,                    &
-       elecselfen, phonselfen, use_ws, nc,                                     &
-       rand_q, rand_nq, rand_k, rand_nk, specfun_pl,                           &
-       nqf1, nqf2, nqf3, nkf1, nkf2, nkf3,                                     &
-       mp_mesh_k, mp_mesh_q, filqf, filkf, ephwrite,                           &
-       band_plot, degaussq, delta_qsmear, nqsmear, nqstep,                     &
-       nswfc, nswc, nswi, pwc, wsfc, wscut, system_2d,                         &
-       broyden_beta, broyden_ndim, nstemp, temps,                              &
-       conv_thr_raxis, conv_thr_iaxis, conv_thr_racon,                         &
+       amass, outdir, prefix, iverbosity, fildvscf, rand_q, rand_nq, rand_k,   &
+       elph, nq1, nq2, nq3, nk1, nk2, nk3, nbndsub, rand_nk, specfun_pl, nswc, &
+       filukk, epbread, epbwrite, epwread, epwwrite, etf_mem, nswfc,    &
+       eig_read, wepexst, epexst, vme, elecselfen, phonselfen, use_ws, nc,     &
+       degaussw, fsthick, nsmear, delta_smear, nqf1, nqf2, nqf3, nkf1, nkf2,   &
+       dvscf_dir, ngaussw, epmatkqread, selecqread, nkf3, mp_mesh_k, mp_mesh_q,&
+       wannierize, dis_win_max, dis_win_min, dis_froz_min, dis_froz_max, nswi, &
+       num_iter, proj, bands_skipped, wdata, iprint, write_wfn, ephwrite,      &
+       wmin, wmax, nw, eps_acustic, a2f, nest_fn, plselfen, filqf, filkf,      &
+       band_plot, fermi_plot, degaussq, delta_qsmear, nqsmear, nqstep, pwc,    &
+       broyden_beta, broyden_ndim, nstemp, temps, bfieldx, bfieldy, bfieldz,   &
+       conv_thr_raxis, conv_thr_iaxis, conv_thr_racon, wsfc, wscut, system_2d, &
        gap_edge, nsiter, muc, lreal, limag, lpade, lacon, liso, laniso, lpolar,&
-       lscreen, scr_typ, fermi_diff, smear_rpa, cumulant, bnd_cum,             &
+       npade, lscreen, scr_typ, fermi_diff, smear_rpa, cumulant, bnd_cum,      &
        lifc, asr_typ, lunif, kerwrite, kerread, imag_read, eliashberg,         &
        ep_coupling, fila2f, max_memlt, efermi_read, fermi_energy,              &
        specfun_el, specfun_ph, wmin_specfun, wmax_specfun, nw_specfun,         &
@@ -166,6 +161,7 @@
        scdm_sigma, assume_metal, wannier_plot, wannier_plot_list, reduce_unk,  &
        wannier_plot_supercell, wannier_plot_scale, wannier_plot_radius,        &
        fixsym, epw_no_t_rev, epw_tr, epw_nosym, epw_noinv, epw_crysym,         &
+       tc_linear, tc_linear_solver,                                            &
   !---------------------------------------------------------------------------------
   ! Added for polaron calculations. Originally by Danny Sio, modified by Chao Lian.
   ! Shell implementation for future use.
@@ -178,46 +174,42 @@
        sigma_edos_plrn, sigma_pdos_plrn, pmax_plrn, pmin_plrn
   ! --------------------------------------------------------------------------------
   !
-  ! tphases, fildvscf0
-  !
   ! amass    : atomic masses
-  ! iverbosity   : verbosity control
+  ! iverbosity : verbosity control
   ! outdir   : directory where input, output, temporary files reside
   ! elph     : if true calculate electron-phonon coefficients
   ! prefix   : the prefix of files produced by pwscf
   ! fildvscf : output file containing deltavsc
   ! fildrho  : output file containing deltarho
   !
-  ! added by @ FG
-  !
-  ! ngaussw  : smearing type after wann interp
-  !            (n >= 0) : Methfessel-Paxton case. See PRB 40, 3616 (1989)
-  !            (n=-1)   : Cold smearing See PRL 82, 3296 (1999)
-  !            (n=-99)  : Fermi-Dirac case: 1.0/(1.0+exp(-x)).
-  !                       If n = -99 you probably want assume_metal == .true. as well.
-  ! degaussw : corresponding width (units of eV)
-  ! filqf    : file with fine q kmesh for interpolation
-  ! filkf    : file with fine kmesh for interpolation
-  ! filukk   : file with rotation matrix U(k) for interpolation
-  ! tphases  : if true set absolute unitary gauge for eigenvectors
-  ! epstrict : if true use strict selection rule for phonon linewidht calculation
-  ! fsthick  : the thickness of the Fermi shell for averaging the e-ph matrix elements (units of eV)
-  ! eptemp   : temperature for the electronic Fermi occupations in the e-p calculation (units of Kelvin)
-  ! fildvscf0: file containing deltavscf to be used as fake perturbation to set phases
-  ! nw       : nr. of bins for frequency scan in \delta( e_k - e_k+q - w ) (units of eV)
-  ! wmin     : min frequency for frequency scan in \delta( e_k - e_k+q - w ) (units of eV)
-  ! wmax     : max    "  "  "                                    (units of eV)
-  ! nbndsub  : number of bands in the optimal subspace (when disentanglement is used)
-  ! elecselfen: if .TRUE. calculate imaginary part of electron selfenergy due to e-p interaction
-  ! phonselfen: if .TRUE. calculate imaginary part of phonon selfenergy due to e-p interaction
-  ! dvscf_dir: the dir containing all the .dvscf and .dyn files
-  ! epbread  : read epmatq array from .epb files
-  ! epbwrite : write epmatq array to .epb files
-  ! epwread  : read all quantities in Wannier representation from file epwdata.fmt
-  ! epwwrite : write all quantities in Wannier representation to file epwdata.fmt
-  !
-  !  added by @jn
+  ! Added by Feliciano Giustino
+  ! ngaussw    : smearing type after wann interp
+  !              (n >= 0) : Methfessel-Paxton case. See PRB 40, 3616 (1989)
+  !              (n=-1)   : Cold smearing See PRL 82, 3296 (1999)
+  !              (n=-99)  : Fermi-Dirac case: 1.0/(1.0+exp(-x)).
+  !                         If n = -99 you probably want assume_metal == .true. as well.
+  ! degaussw   : corresponding width (units of eV)
+  ! filqf      : file with fine q kmesh for interpolation
+  ! filkf      : file with fine kmesh for interpolation
+  ! filukk     : file with rotation matrix U(k) for interpolation
+  ! tphases    : if true set absolute unitary gauge for eigenvectors
+  ! epstrict   : if true use strict selection rule for phonon linewidht calculation
+  ! fsthick    : the thickness of the Fermi shell for averaging the e-ph matrix elements (units of eV)
+  ! eptemp     : temperature for the electronic Fermi occupations in the e-p calculation (units of Kelvin)
+  ! fildvscf0  : file containing deltavscf to be used as fake perturbation to set phases
+  ! nw         : nr. of bins for frequency scan in \delta( e_k - e_k+q - w ) (units of eV)
+  ! wmin       : min frequency for frequency scan in \delta( e_k - e_k+q - w ) (units of eV)
+  ! wmax       : max    "  "  "                                    (units of eV)
+  ! nbndsub    : number of bands in the optimal subspace (when disentanglement is used)
+  ! elecselfen : if .TRUE. calculate imaginary part of electron selfenergy due to e-p interaction
+  ! phonselfen : if .TRUE. calculate imaginary part of phonon selfenergy due to e-p interaction
+  ! dvscf_dir  : the dir containing all the .dvscf and .dyn files
+  ! epbread    : read epmatq array from .epb files
+  ! epbwrite   : write epmatq array to .epb files
+  ! epwread    : read all quantities in Wannier representation from file epwdata.fmt
+  ! epwwrite   : write all quantities in Wannier representation to file epwdata.fmt
   !
+  ! Added by jn
   ! wannierize   : if .TRUE. run the wannier90 code to maximally localize the WFs
   ! dis_win_min  : lower bound on wannier90 disentanglement window
   ! dis_win_max  : upper bound on wannier90 disentanglement window
@@ -238,48 +230,48 @@
   ! eig_read     : if .TRUE. then readin a set of electronic eigenvalues in eV to replace the calcualted ones
   ! wepexst      : if .TRUE. prefix.epmatwe files are already on disk. don't recalculate. debugging param
   ! epexst       : if .TRUE. prefix.epmatwp files are already on disk. don't recalculate  debugging param
-  ! vme          : if .TRUE. calculate velocity matrix elements
-  ! nest_fn      : if true, calculate the nesting function for a given set of q's
+  ! nest_fn      : if .TRUE., calculate the nesting function for a given set of q's
   ! nsmear       : number of smearing values to use for the selfen_phon call
   ! delta_smear  : change in energy for each additional nsmear ( units of eV)
   !
-  ! added by @ RM
-  !
-  ! ephwrite    : if true write el-phonon matrix elements on the fine mesh to file
+  ! Added by Roxana Margine
+  ! ephwrite    : if .TRUE. write el-phonon matrix elements on the fine mesh to file
   ! eps_acustic : min phonon frequency for e-p and a2f calculations (units of cm-1)
-  ! band_plot   : if true write files to plot band structure and phonon dispersion
+  ! band_plot   : if .TRUE. write files to plot band structure and phonon dispersion
+  ! fermi_plot  : if .TRUE. write files to plot Fermi surface
   ! degaussq    : smearing for sum over q in e-ph coupling (units of meV)
   ! delta_qsmear: change in energy for each additional smearing in the a2f (units of meV)
   ! nqsmear     : number of smearings used to calculate a2f
-  ! nqstep   : number of bins for frequency used to calculate a2f
-  ! nswfc    : nr. of grid points between (0,wsfc) in Eliashberg equations
-  ! nswc     : nr. of grid points between (wsfc,wscut)
-  ! pwc      : power used to define nswc for non-uniform grid real-axis calculations
-  ! wsfc     : intermediate freqeuncy used for integration in Eliashberg equations (at least 2-3 times wsphmax)
-  ! wscut    : upper limit for frequency integration in Eliashberg equations (at least 5 times wsphmax) (units of eV)
+  ! nqstep      : number of bins for frequency used to calculate a2f
+  ! nswfc       : nr. of grid points between (0,wsfc) in Eliashberg equations
+  ! nswc        : nr. of grid points between (wsfc,wscut)
+  ! pwc         : power used to define nswc for non-uniform grid real-axis calculations
+  ! wsfc        : intermediate freqeuncy used for integration in Eliashberg equations (at least 2-3 times wsphmax)
+  ! wscut       : upper limit for frequency integration in Eliashberg equations (at least 5 times wsphmax) (units of eV)
   ! broyden_beta : mixing factor for broyden mixing
   ! broyden_ndim : number of iterations used in mixing scheme
-  ! nstemp   : number of temperature points for which the Eliashberg equations are solved
-  ! tempsmin : minimum temperature for which the Eliashberg equations are solved
-  ! tempsmax : maximum temperature " "
+  ! nstemp      : number of temperature points for which the Eliashberg equations are solved
+  ! tempsmin    : minimum temperature for which the Eliashberg equations are solved
+  ! tempsmax    : maximum temperature " "
   ! conv_thr_raxis : convergence threshold for iterative solution of real-axis Eliashberg equations
   ! conv_thr_iaxis : convergence threshold for iterative solution of imag-axis Eliashberg equations
   ! conv_thr_racon : convergence threshold for iterative solution of analytic continuation of
   !                  Eliashberg equations from imag- to real-axis
-  ! gap_edge : initial guess of the superconducting gap (in eV)
-  ! nsiter   : nr of iterations for self-consitency cycle
-  ! muc     : effective Coulomb potential
-  ! lreal   : if .TRUE. solve the real-axis Eliashberg eqautions
-  ! limag   : if .TRUE. solve the imag-axis Eliashberg eqautions
-  ! lpade   : if .TRUE. use pade approximants to continue imag-axis
-  !           Eliashberg equtions to real-axis
-  ! lacon   : if .TRUE. use analytic continuation to continue imag-axis
-  !           Eliashberg equtions to real-axis
-  ! liso    : if .TRUE. solve isotropic case
-  ! laniso  : if .TRUE. solve anisotropic case
-  ! lunif   : if .TRUE. a uniform grid is defined between wsfc and wscut for real-axis calculations
-  ! kerwrite: if .TRUE. write kp and km to files .ker for real-axis calculations
-  ! kerread : if .TRUE. read kp and km from files .ker for real-axis calculations
+  ! gap_edge     : initial guess of the superconducting gap (in eV)
+  ! nsiter       : nr of iterations for self-consitency cycle
+  ! npade        : percentage of Matsubara points used in Pade continuation
+  ! muc          : effective Coulomb potential
+  ! lreal        : if .TRUE. solve the real-axis Eliashberg eqautions
+  ! limag        : if .TRUE. solve the imag-axis Eliashberg eqautions
+  ! lpade        : if .TRUE. use pade approximants to continue imag-axis
+  !                Eliashberg equtions to real-axis
+  ! lacon        : if .TRUE. use analytic continuation to continue imag-axis
+  !                Eliashberg equtions to real-axis
+  ! liso         : if .TRUE. solve isotropic case
+  ! laniso       : if .TRUE. solve anisotropic case
+  ! lunif        : if .TRUE. a uniform grid is defined between wsfc and wscut for real-axis calculations
+  ! kerwrite     : if .TRUE. write kp and km to files .ker for real-axis calculations
+  ! kerread      : if .TRUE. read kp and km from files .ker for real-axis calculations
   ! imag_read    : if .TRUE. read from files Delta and Znorm on the imaginary-axis
   ! eliashberg   : if .TRUE. solve the Eliashberg equations
   ! ep_coupling  : if .TRUE. run e-p coupling calculation
@@ -293,12 +285,12 @@
   ! system_2d    : if .TRUE. two-dimensional system (vaccum is in z-direction)
   ! delta_approx : if .TRUE. the double delta approximation is used to compute the phonon self-energy
   !
-  ! added by CV & SP
-  ! lpolar  : if .TRUE. enable the correct Wannier interpolation in the case of polar material.
-  ! lifc    : if .TRUE. reads interatomic force constants produced by q2r.x for phonon interpolation
-  ! asr_typ : select type of ASR if lifc=.TRUE. (as in matdyn); otherwise it is the usual simple sum rule
-  ! lscreen : if .TRUE. the e-ph matrix elements are screened by the RPA or TF dielectric function
-  ! scr_typ : if 0 calculates the Lindhard screening, if 1 the Thomas-Fermi screening
+  ! Added by Carla Verdi & Samuel Pon\'e
+  ! lpolar     : if .TRUE. enable the correct Wannier interpolation in the case of polar material.
+  ! lifc       : if .TRUE. reads interatomic force constants produced by q2r.x for phonon interpolation
+  ! asr_typ    : select type of ASR if lifc=.TRUE. (as in matdyn); otherwise it is the usual simple sum rule
+  ! lscreen    : if .TRUE. the e-ph matrix elements are screened by the RPA or TF dielectric function
+  ! scr_typ    : if 0 calculates the Lindhard screening, if 1 the Thomas-Fermi screening
   ! fermi_diff : difference between Fermi energy and band edge (in eV)
   ! smear_rpa  : smearing for the calculation of the Lindhard function (in eV)
   ! cumulant   : if .TRUE. calculates the electron spectral function using the cumulant expansion method
@@ -306,9 +298,7 @@
   ! bnd_cum    : band index for which the cumulant calculation is done
   !              (for more than one band, perform multiple calculations and add the results together)
   !
-  !
-  ! Added by SP
-  !
+  ! Added by Samuel Ponc\'e
   ! specfun_el      : if .TRUE. calculate electron spectral function due to e-p interaction
   ! specfun_ph      : if .TRUE. calculate phonon spectral function due to e-p interaction
   ! specfun_pl      : if .TRUE. calculate plason spectral function
@@ -346,6 +336,7 @@
   ! epmatkqread     : If .TRUE., restart an IBTE calculation from scattering written to files.
   ! selecqread      : If .TRUE., restart from the selecq.fmt file
   ! nc              : Number of carrier for the Ziman resistivity formula (can be fractional)
+  ! bfieldx, y, z   : Value of the magnetic field in Tesla along x, y, z direction.
   !
   ! Added by Manos Kioupakis
   ! omegamin        : Photon energy minimum
@@ -357,7 +348,7 @@
   ! Added by Felix Goudreault
   ! assume_metal     : If .TRUE. => we are dealing with a metal
   !
-  ! Added by HL
+  ! Added by Hyungjun Lee
   ! wannier_plot           : If .TRUE., plot Wannier functions
   ! wannier_plot_list      : Field read for parsing Wannier function list
   ! wannier_plot_supercell : Size of supercell for plotting Wannier functions
@@ -365,31 +356,46 @@
   ! wannier_plot_radius    : Cut-off radius for plotting Wannier functions
   ! reduce_unk             : If .TRUE., plot Wannier functions on reduced grids
   !
+  ! Added by Samuel Ponc\'e, Hyungjun Lee and Roxana Margine
+  ! vme : if 'dipole' then computes the velocity as dipole+commutator = <\psi_mk|p/m+i[V_NL,r]|\psi_nk>
+  !     : if 'wannier' then computes the velocity as dH_nmk/dk - i(e_nk-e_mk)A_nmk where A is the Berry connection
+  !     : Note: Before v5.4, vme = .FALSE. was the velocity in the local approximation as <\psi_mk|p|\psi_nk>
+  !             Before v5.4, vme = .TRUE. was = to 'wannier'
+  !
+  !
+  IF ( npool * nimage /= nproc ) THEN
+    CALL errore("epw_readin", "Number of processes must be equal to product "//&
+                "of number of pools and"//new_line('n')//"     number of "//&
+                "images"//new_line('n')//"     Image parallelization can be used "//&
+                "only in calculations on coarse grid.", 1)
+  END IF
+  !
   nk1tmp = 0
   nk2tmp = 0
   nk3tmp = 0
   !
   IF (meta_ionode) THEN
     !
-    ! ... Input from file ?
-    CALL input_from_file()
+    ! ... Input from file (ios=0) or standard input (ios=-1) on unit "qestdin"
+    !
+    ios = open_input_file (  )
     !
     ! ... Read the first line of the input file
     !
-    READ(unit_loc, '(A)', IOSTAT = ios) title
+    IF ( ios <= 0 ) READ(qestdin, '(A)', IOSTAT = ios) title
     !
   ENDIF
   !
   CALL mp_bcast(ios, meta_ionode_id, world_comm)
-  CALL errore('epw_readin', 'reading title ', ABS(ios))
+  CALL errore('epw_readin', 'reading input file ', ABS(ios))
   CALL mp_bcast(title, meta_ionode_id, world_comm)
   !
   ! Rewind the input if the title is actually the beginning of inputph namelist
   !
   IF (imatches("&inputepw", title)) THEN
-    WRITE(*, '(6x,a)') "Title line not specified: using 'default'."
+    WRITE(stdout, '(6x,a)') "Title line not specified: using 'default'."
     title = 'default'
-    IF (meta_ionode) REWIND(unit_loc, IOSTAT = ios)
+    IF (meta_ionode) REWIND(qestdin, IOSTAT = ios)
     CALL mp_bcast(ios, meta_ionode_id, world_comm  )
     CALL errore('epw_readin', 'Title line missing from input.', ABS(ios))
   ENDIF
@@ -477,85 +483,88 @@
   nbndsub      = 0
   nsmear       = 1
   delta_smear  = 0.01d0 ! eV
-  modenum = 0 ! Was -1 previously and read from Modules/input_parameters.f90
-              ! INTEGER :: modenum = 0. In QE 5, modenum variable does not exist
-              ! anymore. Change the default EPW value to match the previous QE
-              ! one.
-  vme = .TRUE. ! Was false by default until EPW 5.1
-  ephwrite = .FALSE.
-  band_plot = .FALSE.
-  nqsmear = 10
-  nqstep = 500
+  modenum      = 0 ! Was -1 previously and read from Modules/input_parameters.f90
+                   ! INTEGER :: modenum = 0. In QE 5, modenum variable does not exist
+                   ! anymore. Change the default EPW value to match the previous QE one.
+  vme          = 'wannier' ! Note: Was .FALSE. by default until EPW v5.1 and then .TRUE. until EPW v5.4
+  ephwrite     = .FALSE.
+  band_plot    = .FALSE.
+  fermi_plot   = .FALSE.
+  nqsmear      = 10
+  nqstep       = 500
   delta_qsmear = 0.05d0 ! meV
-  degaussq = 0.05d0 ! meV
-  lreal   = .FALSE.
-  lunif   = .TRUE.
-  limag   = .FALSE.
-  lpade   = .FALSE.
-  lacon   = .FALSE.
-  liso    = .FALSE.
-  laniso  = .FALSE.
-  lpolar  = .FALSE.
-  lifc    = .FALSE.
-  asr_typ = 'simple'
-  lscreen = .FALSE.
-  scr_typ = 0
-  fermi_diff  = 1.d0
-  smear_rpa   = 0.05d0
-  cumulant    = .FALSE.
-  bnd_cum     = 1
-  kerwrite    = .FALSE.
-  kerread     = .FALSE.
-  imag_read   = .FALSE.
-  eliashberg  = .FALSE.
-  ep_coupling = .TRUE.
-  nswfc    = 0
-  nswc     = 0
-  nswi     = 0
-  pwc      = 1.d0
-  wscut    = 0.d0
-  wsfc     = 0.5d0 * wscut
+  degaussq     = 0.05d0 ! meV
+  lreal        = .FALSE.
+  lunif        = .TRUE.
+  limag        = .FALSE.
+  lpade        = .FALSE.
+  lacon        = .FALSE.
+  liso         = .FALSE.
+  laniso       = .FALSE.
+  lpolar       = .FALSE.
+  lifc         = .FALSE.
+  asr_typ      = 'simple'
+  lscreen      = .FALSE.
+  scr_typ      = 0
+  fermi_diff   = 1.d0
+  smear_rpa    = 0.05d0
+  cumulant     = .FALSE.
+  bnd_cum      = 1
+  kerwrite     = .FALSE.
+  kerread      = .FALSE.
+  imag_read    = .FALSE.
+  eliashberg   = .FALSE.
+  ep_coupling  = .TRUE.
+  tc_linear    = .FALSE.
+  tc_linear_solver = 'power'
+  nswfc        = 0
+  nswc         = 0
+  nswi         = 0
+  pwc          = 1.d0
+  wscut        = 0.d0
+  wsfc         = 0.5d0 * wscut
   broyden_beta = 0.7d0
   broyden_ndim = 8
   conv_thr_raxis = 5.d-04
   conv_thr_iaxis = 1.d-05
   conv_thr_racon = 5.d-04
-  gap_edge = 0.d0
-  nstemp   = 0
-  temps(:) = 0.d0
-  nsiter   = 40
-  muc     = 0.d0
-  fila2f  = ' '
-  max_memlt = 2.85d0
-  efermi_read = .FALSE.
+  gap_edge     = 0.d0
+  nstemp       = 0
+  temps(:)     = 0.d0
+  nsiter       = 40
+  npade        = 90
+  muc          = 0.d0
+  fila2f       = ' '
+  max_memlt    = 2.85d0
+  efermi_read  = .FALSE.
   fermi_energy = 0.d0
   wmin_specfun = 0.d0 ! eV
   wmax_specfun = 0.3d0 ! eV
-  nw_specfun = 100
-  system_2d = .FALSE.
-  scattering = .FALSE.
+  nw_specfun   = 100
+  system_2d    = .FALSE.
+  scattering   = .FALSE.
   scattering_serta = .FALSE.
-  scatread = .FALSE.
+  scatread     = .FALSE.
   scattering_0rta = .FALSE.
-  int_mob    = .FALSE.
+  int_mob      = .FALSE.
   iterative_bte = .FALSE.
-  scissor    = 0.d0 ! eV
-  carrier    = .FALSE.
-  ncarrier   = 0.d0 ! cm^-3
-  longrange  = .FALSE.
-  shortrange = .FALSE.
-  prtgkk     = .FALSE.
-  nel        = 0.0d0
-  meff       = 1.d0
-  epsiheg    = 1.d0
-  lphase     = .FALSE.
-  omegamin   = 0.d0  ! eV
-  omegamax   = 10.d0 ! eV
-  omegastep  = 1.d0  ! eV
-  n_r        = 1.d0
-  lindabs    = .FALSE.
-  mob_maxiter= 50
-  use_ws     = .FALSE.
+  scissor      = 0.d0 ! eV
+  carrier      = .FALSE.
+  ncarrier     = 0.d0 ! cm^-3
+  longrange    = .FALSE.
+  shortrange   = .FALSE.
+  prtgkk       = .FALSE.
+  nel          = 0.0d0
+  meff         = 1.d0
+  epsiheg      = 1.d0
+  lphase       = .FALSE.
+  omegamin     = 0.d0  ! eV
+  omegamax     = 10.d0 ! eV
+  omegastep    = 1.d0  ! eV
+  n_r          = 1.d0
+  lindabs      = .FALSE.
+  mob_maxiter  = 50
+  use_ws       = .FALSE.
   epmatkqread  = .FALSE.
   selecqread   = .FALSE.
   nc           = 4.0d0
@@ -566,6 +575,9 @@
   epw_nosym    = .FALSE.
   epw_noinv    = .FALSE.
   epw_crysym   = .FALSE.
+  bfieldx      = 0.d0  ! Tesla
+  bfieldy      = 0.d0  ! Tesla
+  bfieldz      = 0.d0  ! Tesla
   !
   ! --------------------------------------------------------------------------------
   ! Added for polaron calculations. Originally by Danny Sio, modified by Chao Lian.
@@ -612,19 +624,20 @@
   !
   ! Reading the namelist inputepw and check
   IF (meta_ionode) THEN
-    READ(unit_loc, inputepw, IOSTAT = ios)
+    READ(qestdin, inputepw, IOSTAT = ios)
     ios2 = 0
     IF (ios /= 0) THEN
-      BACKSPACE(5)
-      READ(unit_loc, '(A512)', IOSTAT = ios2) line      
+      BACKSPACE(qestdin)
+      READ(qestdin, '(A512)', IOSTAT = ios2) line
     ENDIF
     IF (ios2 /= 0) CALL errore('epw_readin', 'Could not find namelist &inputepw', 2)
-    IF (ios /= 0) THEN         
+    IF (ios /= 0) THEN
       CALL errore('epw_readin', 'Bad line in namelist &inputepw'&
                  ': "'//TRIM(line)//'" (error could be in the previous line)', 1)
-    ENDIF          
-  ENDIF ! meta_ionode         
-  ! 
+    ENDIF
+    ios = close_input_file ( )
+  ENDIF ! meta_ionode
+  !
   IF (meta_ionode) THEN
     IF (wannier_plot) THEN
       IF (wannier_plot_radius < 0.0d0) &
@@ -741,16 +754,25 @@
       'Error: longrange or shortrange can only be true if lpolar is true as well.', 1)
   IF (longrange .AND. shortrange) CALL errore('epw_readin',&
       'Error: longrange and shortrange cannot be both true.', 1)
-  IF (epwread .AND. .NOT. kmaps .AND. .NOT. epbread) CALL errore('epw_readin', &
-      'Error: kmaps has to be true for a restart run. ', 1)
   IF (.NOT. epwread .AND. .NOT. epwwrite) CALL errore('epw_readin', &
       'Error: Either epwread or epwwrite needs to be true. ', 1)
   IF (lscreen .AND. etf_mem == 2) CALL errore('epw_readin', 'Error: lscreen not implemented with etf_mem=2', 1)
   IF (ABS(degaussw) < eps16 .AND. etf_mem == 2) CALL errore('epw_readin', &
       'Error: adapt_smearing not implemented with etf_mem=2', 1)
-  ! 
+  IF (etf_mem == 3) THEN
+    IF (.NOT. mp_mesh_k) CALL errore('epw_readin', 'When etf_mem == 3, you have to use mp_mesh_k == .true.', 1)
+    IF (.NOT. efermi_read) CALL errore('epw_readin', 'When etf_mem == 3, you have to use efermi_read == .true.', 1)
+    IF (int_mob) CALL errore('epw_readin', 'When etf_mem == 3, you have to use int_mob == .false.', 1)
+    IF (.NOT. carrier) CALL errore('epw_readin', 'When etf_mem == 3, you have to use carrier == .true.', 1)
+    IF (phonselfen) CALL errore('epw_readin', 'phonselfen is not implemented with etf_mem == 3', 1)
+    IF (filkf /= ' ' .OR. filqf /= ' ' .OR. rand_k .OR. rand_q) THEN
+      CALL errore('epw_readin', 'etf_mem == 3 requires homogeneous grids', 1)
+    ENDIF
+  ENDIF
+  !
+  IF (etf_mem > 3 .OR. etf_mem < 0) CALL errore('epw_readin', 'etf_mem can only be 0, 1, 2 or 3.', 1)
   ! Make sure the files exists
-  ! 
+  !
   IF (meta_ionode) THEN
     IF (filkf /= ' ') THEN
       OPEN(UNIT = iunkf, FILE = filkf, STATUS = 'old', FORM = 'formatted', ERR = 100, IOSTAT = ios)
@@ -780,7 +802,33 @@
     WRITE(stdout, '(5x,a)') "         to control the lower bound of band manifold."
   ENDIF
   !
-  ! Setup temperature array
+  ! 2D interpolation - currently experimental feature
+  IF (system_2d) THEN
+    WRITE(stdout, '(5x,a)') "WARNING - 2D - WARNING - 2D - WARNING - 2D - WARNING - 2D"
+    WRITE(stdout, '(5x,a)') "The use of 2D interpolation is experimental for now"
+    WRITE(stdout, '(5x,a)') "Use with caution"
+    WRITE(stdout, '(5x,a)') "WARNING - 2D - WARNING - 2D - WARNING - 2D - WARNING - 2D"
+    CALL errore('epw_readin', '2D interpolation is under developement.', 1)
+  ENDIF
+  !
+  IF (etf_mem == 3) THEN
+    WRITE(stdout,'(5x,a)') 'WARNING: The use of etf_mem == 3 has been tested and validated for cubic and hexagonal materials.'
+    WRITE(stdout,'(5x,a)') '         For other materials, use with care and possibly use etf_mem == 1.'
+  ENDIF
+  !
+  b_abs = ABS(bfieldx) + ABS(bfieldy) + ABS(bfieldz)
+  IF (b_abs > eps20 .AND. (.NOT. mp_mesh_k)) THEN
+    WRITE(stdout,'(5x,a)') 'WARNING: Finite magnetic field is much more stable with k-point symmetry: mp_mesh_k == .true.'
+    CALL errore('epw_readin', 'Error: Finite magnetic field only implemented with k-point symmetry: mp_mesh_k == .true.', 1)
+  ENDIF
+  IF (b_abs > eps20 .AND. (filkf /= ' ')) THEN
+    CALL errore('epw_readin', 'Error: Finite magnetic field only implemented with homogeneous k-grids', 1)
+  ENDIF
+  IF (b_abs > eps20 .AND. (filqf /= ' ')) THEN
+    CALL errore('epw_readin', 'Error: Finite magnetic field only implemented with homogeneous k-grids', 1)
+  ENDIF
+  !
+  ! setup temperature array
   DO itemp = 1, ntempxx
     IF (temps(itemp) > 0.d0) THEN
       nstemp_hold = itemp
@@ -790,12 +838,12 @@
   ! Case of nstemp > 0 but temps(:) = 0 is caught above
   IF (nstemp_hold == 0 .AND. nstemp == 0) THEN !default mode (nstemp_hold == 0 if temps(:) = 0)
     nstemp = 1
-    temps(1) = 300    
+    temps(1) = 300
     WRITE(stdout, '(/,5x,a)') 'No temperature supplied. Setting temps(:) to 300 K.'
   ELSE IF (nstemp == 0 .OR. nstemp_hold == nstemp) THEN !list mode
     nstemp = nstemp_hold !catches if nstemp not supplied, no effect if it is
     WRITE(stdout, '(/,5x,a)') 'Reading supplied temperature list.'
-  ELSE IF (nstemp_hold < nstemp .AND. nstemp_hold == 2) THEN !even spacing mode 
+  ELSE IF (nstemp_hold < nstemp .AND. nstemp_hold == 2) THEN !even spacing mode
     tempsmin = temps(1)
     tempsmax = temps(2)
     IF (tempsmin >= tempsmax) THEN !bad start and end points
@@ -844,6 +892,11 @@
   IF (efermi_read) THEN
     fermi_energy = fermi_energy / ryd2ev
   ENDIF
+  ! bfield: input in Tesla=[V/(m^2/s)] , convert in [eV/(Ang^2/s)]
+  bfieldx = bfieldx * electron_si * ang2m**(-2)
+  bfieldy = bfieldy * electron_si * ang2m**(-2)
+  bfieldz = bfieldz * electron_si * ang2m**(-2)
+  !
   ! eptemp : temperature for the electronic Fermi occupations in the e-p calculation (units of Kelvin)
   ! 1 K in eV = 8.6173423e-5
   ! Out-of bound issue with GCC compiler. Multiple Fermi temp is not used anyway.
@@ -869,8 +922,8 @@
   !
   xq(:) = zero
   !
-  tmp_dir = TRIM(outdir)
-  dvscf_dir = TRIM(dvscf_dir) // '/'
+  tmp_dir = trimcheck(outdir)
+  dvscf_dir = trimcheck(dvscf_dir)
   !
   CALL bcast_epw_input()
   !
