@@ -1,4 +1,4 @@
-# LaTeX2HTML 2018 (Released Feb 1, 2018)
+# LaTeX2HTML 2019.2 (Released June 5, 2019)
 # Associate labels original text with physical files.
 
 
@@ -10,16 +10,24 @@ $key = q/SubSec:Examples/;
 $external_labels{$key} = "$URL/" . q|node5.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:para/;
-$external_labels{$key} = "$URL/" . q|node7.html|; 
+1;
+
+
+# LaTeX2HTML 2019.2 (Released June 5, 2019)
+# labels from external_latex_labels array.
+
+
+$key = q/Sec:para/;
+$external_latex_labels{$key} = q|4|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Examples/;
+$external_latex_labels{$key} = q|3.1|; 
+$noresave{$key} = "$nosave";
+
+$key = q/_/;
+$external_latex_labels{$key} = q|<|; 
 $noresave{$key} = "$nosave";
 
 1;
 
-
-# LaTeX2HTML 2018 (Released Feb 1, 2018)
-# labels from external_latex_labels array.
-
-
-1;
-
