@@ -14,9 +14,11 @@ SUBROUTINE iosys()
   !! those in input_parameters, are locally renamed by adding an underscore "_".
   !
   USE kinds,         ONLY : DP
-  USE funct,         ONLY : dft_is_hybrid, dft_has_finite_size_correction, &
-                            set_finite_size_volume, get_dft_short
-  USE funct,         ONLY : set_exx_fraction, set_screening_parameter
+  USE funct,         ONLY : get_dft_short
+  USE xc_lib,        ONLY : xclib_set_exx_fraction, set_screening_parameter, &
+                            xclib_dft_is, xclib_set_finite_size_volume, &
+                            dft_has_finite_size_correction
+  
   USE control_flags, ONLY : adapt_thr, tr2_init, tr2_multi  
   USE constants,     ONLY : autoev, eV_to_kelvin, pi, rytoev, &
                             ry_kbar, amu_ry, bohr_radius_angs, eps8
@@ -38,7 +40,7 @@ SUBROUTINE iosys()
                             press_       => press, &
                             wmass_       => wmass
   !
-  USE ions_base,     ONLY : if_pos, ityp, tau, extfor, &
+  USE ions_base,     ONLY : if_pos, ityp, tau, extfor, atm, &
                             ntyp_ => nsp, &
                             nat_  => nat, &
                             amass, tau_format
@@ -56,17 +58,6 @@ SUBROUTINE iosys()
                               nraise_     => nraise, &
                               refold_pos_ => refold_pos
   !
-  USE fcp_variables, ONLY : lfcpopt_ => lfcpopt, &
-                            lfcpdyn_ => lfcpdyn, &
-                            fcp_mu_ => fcp_mu, &
-                            fcp_mass_ => fcp_mass, &
-                            fcp_temperature, &
-                            fcp_relax_ => fcp_relax, &
-                            fcp_relax_step_ => fcp_relax_step, &
-                            fcp_relax_crit_ => fcp_relax_crit, &
-                            fcp_mdiis_size_ => fcp_mdiis_size, &
-                            fcp_mdiis_step_ => fcp_mdiis_step
-  !
   USE extfield,      ONLY : tefield_  => tefield, &
                             dipfield_ => dipfield, &
                             edir_     => edir, &
@@ -84,8 +75,7 @@ SUBROUTINE iosys()
                             forcefield, &
                             forcegate
   !
-  USE io_files,      ONLY : input_drho, output_drho, &
-                            psfile, tmp_dir, wfc_dir, &
+  USE io_files,      ONLY : psfile, tmp_dir, wfc_dir, &
                             prefix_     => prefix, &
                             pseudo_dir_ => pseudo_dir, &
                             pseudo_dir_cur, restart_dir, &
@@ -172,6 +162,7 @@ SUBROUTINE iosys()
                             tq_smoothing_     => tq_smoothing, &
                             tbeta_smoothing_  => tbeta_smoothing, &
                             ts_vdw_           => ts_vdw, &
+                            mbd_vdw_          => mbd_vdw, &
                             lecrpa_           => lecrpa, &
                             scf_must_converge_=> scf_must_converge, & 
                             treinit_gvecs_    => treinit_gvecs, &  
@@ -201,12 +192,8 @@ SUBROUTINE iosys()
   USE symm_base, ONLY : no_t_rev_ => no_t_rev, nofrac, allfrac, &
                         nosym_ => nosym, nosym_evc_=> nosym_evc
   !
-  USE bfgs_module,   ONLY : bfgs_ndim_        => bfgs_ndim, &
-                            trust_radius_max_ => trust_radius_max, &
-                            trust_radius_min_ => trust_radius_min, &
-                            trust_radius_ini_ => trust_radius_ini, &
-                            w_1_              => w_1, &
-                            w_2_              => w_2
+  USE bfgs_module,   ONLY : init_bfgs
+  !
   USE wannier_new, ONLY :   use_wannier_      => use_wannier, &
                             use_energy_int_   => use_energy_int, &
                             nwan_             => nwan, &
@@ -220,6 +207,10 @@ SUBROUTINE iosys()
 
   USE qmmm,                  ONLY : qmmm_config
 
+  USE fcp_module,            ONLY : fcp_iosys
+
+  USE gcscf_module,          ONLY : gcscf_iosys
+
   USE vlocal,        ONLY : starting_charge_ => starting_charge
   !
   ! ... CONTROL namelist
@@ -230,8 +221,8 @@ SUBROUTINE iosys()
                                pseudo_dir, disk_io, tefield, dipfield, lberry, &
                                gdir, nppstr, wf_collect,lelfield,lorbm,efield, &
                                nberrycyc, efield_cart, lecrpa,                 &
-                               vdw_table_name, memory, max_seconds, tqmmm,     &
-                               efield_phase, gate, max_xml_steps
+                               lfcp, vdw_table_name, memory, max_seconds,      &
+                               tqmmm, efield_phase, gate, max_xml_steps
 
   !
   ! ... SYSTEM namelist
@@ -263,12 +254,11 @@ SUBROUTINE iosys()
                                vdw_corr, london, london_s6, london_rcut, london_c6, &
                                london_rvdw, dftd3_threebody, dftd3_version,   &
                                ts_vdw, ts_vdw_isolated, ts_vdw_econv_thr,     &
+                               mbd_vdw,     &
                                xdm, xdm_a1, xdm_a2, lforcet,                  &
                                one_atom_occupations,                          &
                                esm_bc, esm_efield, esm_w, esm_nfit, esm_a,    &
-                               lfcpopt, lfcpdyn, fcp_mu, fcp_mass, fcp_tempw, & 
-                               fcp_relax, fcp_relax_step, fcp_relax_crit,     &
-                               fcp_mdiis_size, fcp_mdiis_step,                &
+                               lgcscf,                                        &
                                zgate, relaxz, block, block_1, block_2,        &
                                block_height
   !
@@ -321,7 +311,7 @@ SUBROUTINE iosys()
   USE dftd3_qe,              ONLY : dftd3_printout, dftd3_xc, dftd3, dftd3_in
   USE xdm_module,            ONLY : init_xdm, a1i, a2i
   USE tsvdw_module,          ONLY : vdw_isolated, vdw_econv_thr
-  USE us,                    ONLY : spline_ps_ => spline_ps
+  USE uspp_data,             ONLY : spline_ps_ => spline_ps
   !
   USE qexsd_input,           ONLY : qexsd_input_obj
   USE qes_types_module,      ONLY : input_type
@@ -343,7 +333,6 @@ SUBROUTINE iosys()
   LOGICAL  :: exst, parallelfs, domag, stop_on_error
   REAL(DP) :: at_dum(3,3), theta, phi, ecutwfc_pp, ecutrho_pp, V
   CHARACTER(len=256) :: tempfile
-  INTEGER, EXTERNAL  :: find_free_unit
   !
   ! MAIN CONTROL VARIABLES, MD AND RELAX
   !
@@ -1154,6 +1143,18 @@ SUBROUTINE iosys()
      CALL errore( 'iosys', 'unknown mixing ' // trim( mixing_mode ), 1 )
   END SELECT
   !
+  IF ( mixing_beta < 0.0_DP ) THEN
+     !
+     IF ( lgcscf ) THEN
+        ! GC-SCF with ESM-BC2 or ESM-BC3
+        mixing_beta = 0.2_DP
+     ELSE
+        ! default
+        mixing_beta = 0.7_DP
+     END IF
+     !
+  END IF
+  !
   starting_scf_threshold = tr2
   nmix                   = mixing_ndim
   mixing_beta_           = mixing_beta
@@ -1214,7 +1215,6 @@ SUBROUTINE iosys()
         !
         WRITE( stdout, '(/5x,"Reading Hubbard V parameters from the file parameters.in...",/)')
         !
-        tempunit = find_free_unit()
         tempfile = TRIM("parameters.in")
         !
         INQUIRE (file = tempfile, exist = exst)
@@ -1226,7 +1226,7 @@ SUBROUTINE iosys()
         !
         ! Open the file parameters.in and read Hubbard_V from there
         !
-        OPEN( UNIT = tempunit, FILE = tempfile, FORM = 'formatted', STATUS = 'unknown' )
+        OPEN( NEWUNIT = tempunit, FILE = tempfile, FORM = 'formatted', STATUS = 'unknown' )
         READ(tempunit,*)
 10      READ(tempunit,*,END=11) i, j, V
         Hubbard_V(i,j,1) = V
@@ -1310,19 +1310,16 @@ SUBROUTINE iosys()
   nwan_ = nwan
   print_wannier_coeff_ = print_wannier_coeff
   !
-  !
   ! ... BFGS specific
   !
-  bfgs_ndim_        = bfgs_ndim
-  trust_radius_max_ = trust_radius_max
-  trust_radius_min_ = trust_radius_min
-  trust_radius_ini_ = trust_radius_ini
-  w_1_              = w_1
-  w_2_              = w_2
+  CALL init_bfgs( stdout, bfgs_ndim, trust_radius_max, trust_radius_min, &
+        trust_radius_ini, w_1, w_2 )
+  !
+  IF (trim(occupations) /= 'from_input') one_atom_occupations_=.false.
   !
   !  VdW CORRECTIONS (SEMI-EMPIRICAL)
   !
-  CALL set_vdw_corr ( vdw_corr, llondon, ldftd3, ts_vdw_, lxdm)
+  CALL set_vdw_corr ( vdw_corr, llondon, ldftd3, ts_vdw_, mbd_vdw_, lxdm)
   !
   IF ( london ) THEN
      CALL infomsg("iosys","london is obsolete, use ""vdw_corr='grimme-d2'"" instead")
@@ -1339,6 +1336,11 @@ SUBROUTINE iosys()
      vdw_corr='TS'
      ts_vdw_ = .TRUE.
   END IF
+  IF ( mbd_vdw ) THEN
+     CALL infomsg("iosys","mbd_vdw is obsolete, use ""vdw_corr='MBD'"" instead")
+     vdw_corr='MBD'
+     mbd_vdw_ = .TRUE.
+  END IF
   IF ( llondon.AND.lxdm .OR. llondon.AND.ts_vdw_ .OR. lxdm.AND.ts_vdw_ .OR. &
            ldftd3.AND.llondon .OR. ldftd3.AND.lxdm .OR. ldftd3.AND.ts_vdw ) &
      CALL errore("iosys","must choose a unique vdW correction!", 1)
@@ -1378,10 +1380,6 @@ SUBROUTINE iosys()
       IF ( ibrav < 1 .OR. ibrav > 3 ) CALL errore(' iosys', &
               'Makov-Payne correction defined only for cubic lattices', 1)
       !
-    CASE( 'dcc' )
-      !
-      CALL errore('iosys','density countercharge correction currently disabled',1)
-      !
     CASE( 'martyna-tuckerman', 'm-t', 'mt' )
       !
       do_comp_mt     = .true.
@@ -1394,7 +1392,15 @@ SUBROUTINE iosys()
       !
       do_cutoff_2D   = .true.
       !
-
+    CASE ( 'none' )
+      !
+      CONTINUE
+      !
+    CASE DEFAULT
+      !
+      CALL errore('iosys','unknown value assume_isolated="' // &
+              & TRIM(assume_isolated) // '"',1)
+      !
   END SELECT
   !
   IF ( do_comp_mt .AND. lstres ) THEN
@@ -1424,30 +1430,6 @@ SUBROUTINE iosys()
     ENDIF
   ENDIF
   !
-  ! ... FCP
-  !
-  lfcpopt_        = lfcpopt
-  lfcpdyn_        = lfcpdyn
-  fcp_mu_         = fcp_mu
-  fcp_mass_       = fcp_mass
-  fcp_temperature = fcp_tempw
-  !
-  IF ( lfcpopt .or. lfcpdyn ) THEN
-     IF ( .not. do_comp_esm ) THEN
-        CALL errore ('iosys','FCP optimise/dynamics currently not available without ESM',1)
-     ENDIF
-     IF ( trim( calculation ).NE.'relax'.AND.trim( calculation ).NE.'md')THEN
-        CALL errore ('iosys',"FCP optimise/dynamics only available with calculation = 'relax' and 'md'",1)
-     ENDIF
-  ENDIF
-  !
-  IF ( fcp_temperature == 0.0_DP ) &
-     fcp_temperature = temperature
-  fcp_relax_      = fcp_relax
-  fcp_relax_step_ = fcp_relax_step
-  fcp_relax_crit_ = fcp_relax_crit
-  fcp_mdiis_size_ = fcp_mdiis_size
-  fcp_mdiis_step_ = fcp_mdiis_step
   !
   ! ATOMIC POSITIONS
   !
@@ -1615,23 +1597,23 @@ SUBROUTINE iosys()
           'ecutfock can not be < ecutwfc or > ecutrho!', 1) 
      ecutfock_ = ecutfock
   END IF
-  IF ( lstres .AND. dft_is_hybrid() .AND. npool > 1 )  CALL errore('iosys', &
+  IF ( lstres .AND. xclib_dft_is('hybrid') .AND. npool > 1 )  CALL errore('iosys', &
          'stress for hybrid functionals not available with pools', 1)
-  IF ( lmovecell.AND. dft_is_hybrid() ) CALL infomsg('iosys',&
+  IF ( lmovecell.AND. xclib_dft_is('hybrid') ) CALL infomsg('iosys',&
          'Variable cell and hybrid XC little tested')
   !
   ! ... must be done AFTER dft is read from PP files and initialized
   ! ... or else the two following parameters will be overwritten
   !
-  IF (exx_fraction >= 0.0_DP) CALL set_exx_fraction (exx_fraction)
+  IF (exx_fraction >= 0.0_DP) CALL xclib_set_exx_fraction (exx_fraction)
   !
   IF (screening_parameter >= 0.0_DP) &
-        & CALL set_screening_parameter (screening_parameter)
+        & CALL set_screening_parameter(screening_parameter)
   !
   ! ... if DFT finite size corrections are needed, define the appropriate volume
   !
   IF (dft_has_finite_size_correction()) &
-      CALL set_finite_size_volume(REAL(omega*nk1*nk2*nk3))
+      CALL xclib_set_finite_size_volume(REAL(omega*nk1*nk2*nk3))
   !
   ! VARIABLE-CELL DYNAMICS
   !
@@ -1659,7 +1641,8 @@ SUBROUTINE iosys()
       if (dftd3_version==2) dftd3_threebody=.false.
       dftd3_in%threebody = dftd3_threebody
       CALL dftd3_init(dftd3, dftd3_in)
-      CALL dftd3_printout(dftd3, dftd3_in)
+      CALL dftd3_printout(dftd3, dftd3_in, stdout, ntyp, atm, nat, ityp,&
+                  tau, at, alat )
       dft_ = get_dft_short( )
       dft_ = dftd3_xc ( dft_ )
       CALL dftd3_set_functional(dftd3, func=dft_, version=dftd3_version,tz=.false.)
@@ -1677,6 +1660,14 @@ SUBROUTINE iosys()
      CALL init_constraint( nat, tau, ityp, alat )
   END IF
   !
+  ! ... set variables for FCP
+  !
+  CALL fcp_iosys(lfcp)
+  !
+  ! ... set variables for GC-SCF (this must be after FCP, to check condition)
+  !
+  CALL gcscf_iosys(lgcscf)
+  !
   ! ... End of reading input parameters
   !
 #if ! defined (__INTEL_COMPILER) || (__INTEL_COMPILER >= 1300) 
